@@ -1,8 +1,10 @@
+# isort: skip_file
 """
 PySTAC is a library for working with SpatioTemporal Asset Catalogs (STACs)
 """
 __all__ = [
     "__version__",
+    "TemplateError",
     "STACError",
     "STACTypeError",
     "DuplicateObjectKeyError",
@@ -11,6 +13,7 @@ __all__ = [
     "ExtensionTypeError",
     "RequiredPropertyMissing",
     "STACValidationError",
+    "DeprecatedWarning",
     "MediaType",
     "RelType",
     "StacIO",
@@ -40,9 +43,10 @@ __all__ = [
 ]
 
 import os
-from typing import Any, AnyStr, Dict, Optional, Union
+from typing import Any, Dict, Optional
 
 from pystac.errors import (
+    TemplateError,
     STACError,
     STACTypeError,
     DuplicateObjectKeyError,
@@ -51,6 +55,7 @@ from pystac.errors import (
     ExtensionTypeError,
     RequiredPropertyMissing,
     STACValidationError,
+    DeprecatedWarning,
 )
 
 from pystac.version import (
