@@ -1,35 +1,39 @@
+from __future__ import annotations
+
 import os
-from html import escape
 from copy import deepcopy
-from pystac.errors import STACTypeError
-from pystac.html.jinja_env import get_jinja_env
+from html import escape
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
     Iterable,
     List,
     Optional,
-    TYPE_CHECKING,
     Tuple,
+    Type,
+    TypeVar,
     Union,
     cast,
 )
 
 import pystac
-from pystac.stac_object import STACObject, STACObjectType
+from pystac.cache import ResolvedObjectCache
+from pystac.errors import STACTypeError
+from pystac.html.jinja_env import get_jinja_env
 from pystac.layout import (
     BestPracticesLayoutStrategy,
     HrefLayoutStrategy,
     LayoutTemplate,
 )
 from pystac.link import Link
-from pystac.cache import ResolvedObjectCache
 from pystac.serialization import (
-    identify_stac_object_type,
     identify_stac_object,
+    identify_stac_object_type,
     migrate_to_latest,
 )
+from pystac.stac_object import STACObject, STACObjectType
 from pystac.utils import (
     StringEnum,
     is_absolute_href,
@@ -38,9 +42,11 @@ from pystac.utils import (
 )
 
 if TYPE_CHECKING:
-    from pystac.asset import Asset as Asset_Type
-    from pystac.item import Item as Item_Type
-    from pystac.collection import Collection as Collection_Type
+    from pystac.asset import Asset
+    from pystac.collection import Collection
+    from pystac.item import Item
+
+C = TypeVar("C", bound="Catalog")
 
 
 class CatalogType(StringEnum):
@@ -75,7 +81,7 @@ class CatalogType(StringEnum):
     """
 
     @classmethod
-    def determine_type(cls, stac_json: Dict[str, Any]) -> Optional["CatalogType"]:
+    def determine_type(cls, stac_json: Dict[str, Any]) -> Optional[CatalogType]:
         """Determines the catalog type based on a STAC JSON dict.
 
         Only applies to Catalogs or Collections
@@ -85,7 +91,7 @@ class CatalogType(StringEnum):
 
         Returns:
             Optional[CatalogType]: The catalog type of the catalog or collection.
-                Will return None if it cannot be determined.
+            Will return None if it cannot be determined.
         """
         self_link = None
         relative = False
@@ -228,19 +234,20 @@ class Catalog(STACObject):
 
     def add_child(
         self,
-        child: Union["Catalog", "Collection_Type"],
+        child: Union["Catalog", Collection],
         title: Optional[str] = None,
         strategy: Optional[HrefLayoutStrategy] = None,
     ) -> None:
-        """Adds a link to a child :class:`~pystac.Catalog` or :class:`~pystac.Collection`.
-        This method will set the child's parent to this object, and its root to
-        this Catalog's root.
+        """Adds a link to a child :class:`~pystac.Catalog` or
+        :class:`~pystac.Collection`. This method will set the child's parent to this
+        object, and its root to this Catalog's root.
 
         Args:
             child : The child to add.
             title : Optional title to give to the :class:`~pystac.Link`
             strategy : The layout strategy to use for setting the
-                self href of the child.
+                self href of the child. If not provided, defaults to
+                :class:`~pystac.layout.BestPracticesLayoutStrategy`.
         """
 
         # Prevent typo confusion
@@ -261,12 +268,10 @@ class Catalog(STACObject):
 
         self.add_link(Link.child(child, title=title))
 
-    def add_children(
-        self, children: Iterable[Union["Catalog", "Collection_Type"]]
-    ) -> None:
-        """Adds links to multiple :class:`~pystac.Catalog` or `~pystac.Collection` objects.
-        This method will set each child's parent to this object, and their root to
-        this Catalog's root.
+    def add_children(self, children: Iterable[Union["Catalog", Collection]]) -> None:
+        """Adds links to multiple :class:`~pystac.Catalog` or `~pystac.Collection`
+        objects. This method will set each child's parent to this object, and their
+        root to this Catalog's root.
 
         Args:
             children : The children to add.
@@ -276,7 +281,7 @@ class Catalog(STACObject):
 
     def add_item(
         self,
-        item: "Item_Type",
+        item: Item,
         title: Optional[str] = None,
         strategy: Optional[HrefLayoutStrategy] = None,
     ) -> None:
@@ -287,6 +292,9 @@ class Catalog(STACObject):
         Args:
             item : The item to add.
             title : Optional title to give to the :class:`~pystac.Link`
+            strategy : The layout strategy to use for setting the
+                self href of the item. If not provided, defaults to
+                :class:`~pystac.layout.BestPracticesLayoutStrategy`.
         """
 
         # Prevent typo confusion
@@ -307,20 +315,28 @@ class Catalog(STACObject):
 
         self.add_link(Link.item(item, title=title))
 
-    def add_items(self, items: Iterable["Item_Type"]) -> None:
-        """Adds links to multiple :class:`~pystac.Item` s.
+    def add_items(
+        self,
+        items: Iterable[Item],
+        strategy: Optional[HrefLayoutStrategy] = None,
+    ) -> None:
+        """Adds links to multiple :class:`Items <pystac.Item>`.
+
         This method will set each item's parent to this object, and their root to
         this Catalog's root.
 
         Args:
             items : The items to add.
+            strategy : The layout strategy to use for setting the
+                self href of the items. If not provided, defaults to
+                :class:`~pystac.layout.BestPracticesLayoutStrategy`.
         """
         for item in items:
-            self.add_item(item)
+            self.add_item(item, strategy=strategy)
 
     def get_child(
         self, id: str, recursive: bool = False
-    ) -> Optional[Union["Catalog", "Collection_Type"]]:
+    ) -> Optional[Union["Catalog", Collection]]:
         """Gets the child of this catalog with the given ID, if it exists.
 
         Args:
@@ -342,7 +358,7 @@ class Catalog(STACObject):
                     return child
             return None
 
-    def get_children(self) -> Iterable[Union["Catalog", "Collection_Type"]]:
+    def get_children(self) -> Iterable[Union["Catalog", Collection]]:
         """Return all children of this catalog.
 
         Return:
@@ -354,7 +370,7 @@ class Catalog(STACObject):
             self.get_stac_objects(pystac.RelType.CHILD),
         )
 
-    def get_collections(self) -> Iterable["Collection_Type"]:
+    def get_collections(self) -> Iterable[Collection]:
         """Return all children of this catalog that are :class:`~pystac.Collection`
         instances."""
         return map(
@@ -362,7 +378,7 @@ class Catalog(STACObject):
             self.get_stac_objects(pystac.RelType.CHILD, pystac.Collection),
         )
 
-    def get_all_collections(self) -> Iterable["Collection_Type"]:
+    def get_all_collections(self) -> Iterable[Collection]:
         """Get all collections from this catalog and all subcatalogs. Will traverse
         any subcatalogs recursively."""
         yield from self.get_collections()
@@ -408,7 +424,7 @@ class Catalog(STACObject):
                     child.set_root(None)
         self.links = new_links
 
-    def get_item(self, id: str, recursive: bool = False) -> Optional["Item_Type"]:
+    def get_item(self, id: str, recursive: bool = False) -> Optional[Item]:
         """Returns an item with a given ID.
 
         Args:
@@ -429,7 +445,7 @@ class Catalog(STACObject):
                     return item
             return None
 
-    def get_items(self) -> Iterable["Item_Type"]:
+    def get_items(self) -> Iterable[Item]:
         """Return all items of this catalog.
 
         Return:
@@ -474,7 +490,7 @@ class Catalog(STACObject):
                     item.set_root(None)
         self.links = new_links
 
-    def get_all_items(self) -> Iterable["Item_Type"]:
+    def get_all_items(self) -> Iterable[Item]:
         """Get all items from this catalog and all subcatalogs. Will traverse
         any subcatalogs recursively.
 
@@ -498,7 +514,11 @@ class Catalog(STACObject):
     def to_dict(
         self, include_self_link: bool = True, transform_hrefs: bool = True
     ) -> Dict[str, Any]:
-        links = self.links
+        links = [
+            x
+            for x in self.links
+            if x.rel != pystac.RelType.ROOT or x.get_href(transform_hrefs) is not None
+        ]
         if not include_self_link:
             links = [x for x in links if x.rel != pystac.RelType.SELF]
 
@@ -510,7 +530,7 @@ class Catalog(STACObject):
             "links": [link.to_dict(transform_href=transform_hrefs) for link in links],
         }
 
-        if self.stac_extensions is not None:
+        if self.stac_extensions:
             d["stac_extensions"] = self.stac_extensions
 
         for key in self.extra_fields:
@@ -521,7 +541,7 @@ class Catalog(STACObject):
 
         return d
 
-    def clone(self) -> "Catalog":
+    def clone(self) -> Catalog:
         cls = self.__class__
         clone = cls(
             id=self.id,
@@ -568,6 +588,7 @@ class Catalog(STACObject):
         catalog_type: Optional[CatalogType] = None,
         strategy: Optional[HrefLayoutStrategy] = None,
         stac_io: Optional[pystac.StacIO] = None,
+        skip_unresolved: bool = False,
     ) -> None:
         """Normalizes link HREFs to the given root_href, and saves the catalog.
 
@@ -583,29 +604,44 @@ class Catalog(STACObject):
                 Defaults to the root catalog.catalog_type or the current catalog
                 catalog_type if there is no root catalog.
             strategy : The layout strategy to use in setting the
-                HREFS for this catalog. Defaults to
+                HREFS for this catalog. If not provided, defaults to
                 :class:`~pystac.layout.BestPracticesLayoutStrategy`
             stac_io : Optional instance of :class:`~pystac.StacIO` to use. If not
                 provided, will use the instance set while reading in the catalog,
                 or the default instance if this is not available.
+            skip_unresolved : Skip unresolved links when normalizing the tree.
+                Defaults to False. Because unresolved links are not saved, this
+                argument can be used to normalize and save only newly-added
+                objects.
         """
-        self.normalize_hrefs(root_href, strategy=strategy)
+        self.normalize_hrefs(
+            root_href, strategy=strategy, skip_unresolved=skip_unresolved
+        )
         self.save(catalog_type, stac_io=stac_io)
 
     def normalize_hrefs(
-        self, root_href: str, strategy: Optional[HrefLayoutStrategy] = None
+        self,
+        root_href: str,
+        strategy: Optional[HrefLayoutStrategy] = None,
+        skip_unresolved: bool = False,
     ) -> None:
         """Normalize HREFs will regenerate all link HREFs based on
         an absolute root_href and the canonical catalog layout as specified
         in the STAC specification's best practices.
 
-        This method mutates the entire catalog tree.
+        This method mutates the entire catalog tree, unless ``skip_unresolved``
+        is True, in which case only resolved links are modified. This is useful
+        in the case when you have loaded a large catalog and you've added a few
+        items/children, and you only want to update those newly-added objects,
+        not the whole tree.
 
         Args:
             root_href : The absolute HREF that all links will be normalized against.
             strategy : The layout strategy to use in setting the HREFS
-                for this catalog. Defaults to
+                for this catalog. If not provided, defaults to
                 :class:`~pystac.layout.BestPracticesLayoutStrategy`
+            skip_unresolved : Skip unresolved links when normalizing the tree.
+                Defaults to False.
 
         See:
             :stac-spec:`STAC best practices document <best-practices.md#catalog-layout>`
@@ -620,8 +656,9 @@ class Catalog(STACObject):
         if not is_absolute_href(root_href):
             root_href = make_absolute_href(root_href, os.getcwd(), start_is_dir=True)
 
-        def process_item(item: "Item_Type", _root_href: str) -> Callable[[], None]:
-            item.resolve_links()
+        def process_item(item: Item, _root_href: str) -> Callable[[], None]:
+            if not skip_unresolved:
+                item.resolve_links()
 
             new_self_href = _strategy.get_href(item, _root_href)
 
@@ -635,16 +672,29 @@ class Catalog(STACObject):
         ) -> List[Callable[[], None]]:
             setter_funcs: List[Callable[[], None]] = []
 
-            cat.resolve_links()
+            if not skip_unresolved:
+                cat.resolve_links()
 
             new_self_href = _strategy.get_href(cat, _root_href, is_root)
             new_root = os.path.dirname(new_self_href)
 
-            for item in cat.get_items():
-                setter_funcs.append(process_item(item, new_root))
-
-            for child in cat.get_children():
-                setter_funcs.extend(process_catalog(child, new_root, is_root=False))
+            for link in cat.get_links():
+                if skip_unresolved and not link.is_resolved():
+                    continue
+                elif link.rel == pystac.RelType.ITEM:
+                    link.resolve_stac_object(root=self.get_root())
+                    setter_funcs.append(
+                        process_item(cast(pystac.Item, link.target), new_root)
+                    )
+                elif link.rel == pystac.RelType.CHILD:
+                    link.resolve_stac_object(root=self.get_root())
+                    setter_funcs.extend(
+                        process_catalog(
+                            cast(Union[pystac.Catalog, pystac.Collection], link.target),
+                            new_root,
+                            is_root=False,
+                        )
+                    )
 
             def fn() -> None:
                 cat.set_self_href(new_self_href)
@@ -747,8 +797,8 @@ class Catalog(STACObject):
         dest_href: Optional[str] = None,
         stac_io: Optional[pystac.StacIO] = None,
     ) -> None:
-        """Save this catalog and all it's children/item to files determined by the object's
-        self link HREF or a specified path.
+        """Save this catalog and all it's children/item to files determined by the
+        object's self link HREF or a specified path.
 
         Args:
             catalog_type : The catalog type that dictates the structure of
@@ -839,7 +889,7 @@ class Catalog(STACObject):
 
     def walk(
         self,
-    ) -> Iterable[Tuple["Catalog", Iterable["Catalog"], Iterable["Item_Type"]]]:
+    ) -> Iterable[Tuple["Catalog", Iterable["Catalog"], Iterable[Item]]]:
         """Walks through children and items of catalogs.
 
         For each catalog in the STAC's tree rooted at this catalog (including this
@@ -861,6 +911,20 @@ class Catalog(STACObject):
         for child in self.get_children():
             yield from child.walk()
 
+    def fully_resolve(self) -> None:
+        """Resolves every link in this catalog.
+
+        Useful if, e.g., you'd like to read a catalog from a filesystem, upgrade
+        every object in the catalog to the latest STAC version, and save it back
+        to the filesystem. By default, :py:meth:`~pystac.Catalog.save` skips
+        unresolved links.
+        """
+        for _, _, items in self.walk():
+            # items is a generator, so we need to consume it to resolve the
+            # items
+            for item in items:
+                pass
+
     def validate_all(self) -> None:
         """Validates each catalog, collection contained within this catalog.
 
@@ -886,8 +950,8 @@ class Catalog(STACObject):
 
     def map_items(
         self,
-        item_mapper: Callable[["Item_Type"], Union["Item_Type", List["Item_Type"]]],
-    ) -> "Catalog":
+        item_mapper: Callable[[Item], Union[Item, List[Item]]],
+    ) -> Catalog:
         """Creates a copy of a catalog, with each item passed through the
         item_mapper function.
 
@@ -930,10 +994,10 @@ class Catalog(STACObject):
     def map_assets(
         self,
         asset_mapper: Callable[
-            [str, "Asset_Type"],
-            Union["Asset_Type", Tuple[str, "Asset_Type"], Dict[str, "Asset_Type"]],
+            [str, Asset],
+            Union[Asset, Tuple[str, Asset], Dict[str, Asset]],
         ],
-    ) -> "Catalog":
+    ) -> Catalog:
         """Creates a copy of a catalog, with each Asset for each Item passed
         through the asset_mapper function.
 
@@ -949,7 +1013,7 @@ class Catalog(STACObject):
         """
 
         def apply_asset_mapper(
-            tup: Tuple[str, "Asset_Type"]
+            tup: Tuple[str, Asset]
         ) -> List[Tuple[str, pystac.Asset]]:
             k, v = tup
             result = asset_mapper(k, v)
@@ -998,13 +1062,13 @@ class Catalog(STACObject):
 
     @classmethod
     def from_dict(
-        cls,
+        cls: Type[C],
         d: Dict[str, Any],
         href: Optional[str] = None,
         root: Optional["Catalog"] = None,
         migrate: bool = False,
         preserve_dict: bool = True,
-    ) -> "Catalog":
+    ) -> C:
         if migrate:
             info = identify_stac_object(d)
             d = migrate_to_latest(d, info)
@@ -1050,11 +1114,13 @@ class Catalog(STACObject):
 
     def full_copy(
         self, root: Optional["Catalog"] = None, parent: Optional["Catalog"] = None
-    ) -> "Catalog":
+    ) -> Catalog:
         return cast(Catalog, super().full_copy(root, parent))
 
     @classmethod
-    def from_file(cls, href: str, stac_io: Optional[pystac.StacIO] = None) -> "Catalog":
+    def from_file(
+        cls: Type[C], href: str, stac_io: Optional[pystac.StacIO] = None
+    ) -> C:
         if stac_io is None:
             stac_io = pystac.StacIO.default()
 
