@@ -1,6 +1,14 @@
 from typing import Any, Optional, Union
 
 
+class TemplateError(Exception):
+    """Exception thrown when an error occurs during converting a template
+    string into data for :class:`~pystac.layout.LayoutTemplate`
+    """
+
+    pass
+
+
 class STACError(Exception):
     """A STACError is raised for errors relating to STAC, e.g. for
     invalid formats or trying to operate on a STAC that does not have
@@ -79,3 +87,10 @@ class STACValidationError(Exception):
     def __init__(self, message: str, source: Optional[Any] = None):
         super().__init__(message)
         self.source = source
+
+
+class DeprecatedWarning(FutureWarning):
+    """Issued when converting a dictionary to a STAC Item or Collection and the
+    version extension ``deprecated`` field is present and set to ``True``."""
+
+    pass
