@@ -1,16 +1,30 @@
+from __future__ import annotations
+
 from copy import deepcopy
-from pystac.errors import STACTypeError
-from typing import Any, Dict, Iterator, List, Optional, Collection, Iterable, Union
 from html import escape
+from typing import (
+    Any,
+    Collection,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import pystac
+from pystac.errors import STACTypeError
 from pystac.html.jinja_env import get_jinja_env
-from pystac.utils import make_absolute_href, is_absolute_href
 from pystac.serialization.identify import identify_stac_object_type
-
+from pystac.utils import is_absolute_href, make_absolute_href
 
 ItemLike = Union[pystac.Item, Dict[str, Any]]
 
+C = TypeVar("C", bound="ItemCollection")
+
 
 class ItemCollection(Collection[pystac.Item]):
     """Implementation of a GeoJSON FeatureCollection whose features are all STAC
@@ -34,9 +48,9 @@ class ItemCollection(Collection[pystac.Item]):
             :class:`~ItemCollection`.
         clone_items : Optional flag indicating whether :class:`~pystac.Item` instances
             should be cloned before storing in the :class:`~ItemCollection`. Setting to
-            ``True`` ensures that changes made to :class:`~pystac.Item` instances in
-            the :class:`~ItemCollection` will not mutate the original ``Item``, but
-            will result in slower instantiation. Defaults to ``False``.
+            ``False`` will result in faster instantiation, but changes made to
+            :class:`~pystac.Item` instances in the :class:`~ItemCollection` will mutate
+            the original ``Item``. Defaults to ``True``.
 
     Examples:
 
@@ -57,7 +71,7 @@ class ItemCollection(Collection[pystac.Item]):
         object equality (i.e. ``item_1 is item_2``).
 
         >>> item: Item = ...
-        >>> item_collection = ItemCollection(items=[item])
+        >>> item_collection = ItemCollection(items=[item], clone_items=False)
         >>> assert item in item_collection
 
         Combine :class:`~ItemCollection` instances
@@ -68,8 +82,8 @@ class ItemCollection(Collection[pystac.Item]):
         >>> item_collection_1 = ItemCollection(items=[item_1, item_2])
         >>> item_collection_2 = ItemCollection(items=[item_2, item_3])
         >>> combined = item_collection_1 + item_collection_2
-        >>> assert len(combined) == 3
-        # If an item is present in both ItemCollections it will only be added once
+        >>> assert len(combined) == 4
+        # If an item is present in both ItemCollections it will occur twice
     """
 
     items: List[pystac.Item]
@@ -83,7 +97,7 @@ class ItemCollection(Collection[pystac.Item]):
         self,
         items: Iterable[ItemLike],
         extra_fields: Optional[Dict[str, Any]] = None,
-        clone_items: bool = False,
+        clone_items: bool = True,
     ):
         def map_item(item_or_dict: ItemLike) -> pystac.Item:
             # Converts dicts to pystac.Items and clones if necessary
@@ -111,12 +125,8 @@ class ItemCollection(Collection[pystac.Item]):
         if not isinstance(other, ItemCollection):
             return NotImplemented
 
-        combined = []
-        for item in self.items + other.items:
-            if item not in combined:
-                combined.append(item)
-
-        return ItemCollection(items=combined, clone_items=False)
+        combined = [*self.items, *other.items]
+        return ItemCollection(items=combined)
 
     def to_dict(self, transform_hrefs: bool = False) -> Dict[str, Any]:
         """Serializes an :class:`ItemCollection` instance to a JSON-like dictionary.
@@ -138,7 +148,7 @@ class ItemCollection(Collection[pystac.Item]):
             **self.extra_fields,
         }
 
-    def clone(self) -> "ItemCollection":
+    def clone(self) -> ItemCollection:
         """Creates a clone of this instance. This clone is a deep copy; all
         :class:`~pystac.Item` instances are cloned and all additional top-level fields
         are deep copied."""
@@ -149,11 +159,11 @@ class ItemCollection(Collection[pystac.Item]):
 
     @classmethod
     def from_dict(
-        cls,
+        cls: Type[C],
         d: Dict[str, Any],
         preserve_dict: bool = True,
         root: Optional[pystac.Catalog] = None,
-    ) -> "ItemCollection":
+    ) -> C:
         """Creates a :class:`ItemCollection` instance from a dictionary.
 
         Arguments:
@@ -177,8 +187,8 @@ class ItemCollection(Collection[pystac.Item]):
 
     @classmethod
     def from_file(
-        cls, href: str, stac_io: Optional[pystac.StacIO] = None
-    ) -> "ItemCollection":
+        cls: Type[C], href: str, stac_io: Optional[pystac.StacIO] = None
+    ) -> C:
         """Reads a :class:`ItemCollection` from a JSON file.
 
         Arguments:
